@@ -19,26 +19,36 @@
 
 doubledecker_arules <- function(rules, measure ="support", data, 
 	control=list(), ...) {
-    
-    if(length(rules) != 1) stop("only can visualize one rule.")
-    if(is.null(data)) stop("Data missing.")
-    
-    control <- .get_parameters(control, list(
-		    main = "Doubledecker plot for 1 rule",
-		    type = "doubledecker",
-		    interactive = FALSE
-		    ))
 
-    table <- getTable(rules, data)
+  if(pmatch(control$engine, c("default"), nomatch = 0) != 1) 
+    stop("Unknown engine for scatterplot: '", control$engine, 
+      "' Valid engines: 'default'.")
+  
+  if(length(rules) != 1) stop("only can visualize one rule.")
+  if(is.null(data)) stop("data has to be specified, but is missing.")
 
-    if(control$type=="doubledecker")
-    doubledecker(table, margins=c(2,8,length(dim(table) + 2), 2), 
-	    main = control$main, ...) 
-    else {
+  control <- c(control, list(...))
+
+  control <- .get_parameters(control, list(
+    main = "Doubledecker plot for 1 rule",
+    type = "doubledecker",
+    engine = "default",
+    interactive = FALSE,
+    plot_options = list()
+  ))
+
+  if(control$interactive) stop("No interactive visualization available for doubledecker/mosaic plot.")
+    
+  table <- getTable(rules, data)
+  
+  if(control$type=="doubledecker")
+    do.call(doubledecker, c(list(table, margins=c(2,8,length(dim(table) + 2), 2), 
+      main = control$main), control$plot_options)) 
+  else {
     control$main <- "Mosaic plot for 1 rule"
-    mosaic(table, highlighting = length(dim(table)),
-            main = control$main, ...)
-    }
+    do.call(mosaic, c(list(table, highlighting = length(dim(table)),
+      main = control$main), control$plot_options))
+  }
 }
 
 
