@@ -25,6 +25,12 @@ ruleExplorer <- function(x, parameter = NULL) {
   if (!requireNamespace("shiny", quietly = TRUE)) {
     stop("Package shiny is required to run this method.", call. = FALSE)
   }
+  
+  if (!requireNamespace("shinythemes", quietly = TRUE)) {
+    stop("Package shinythemes is required to run this method.", call. = FALSE)
+  }
+  
+  ### rounding helpers
   roundUp <- function(x,digits = 3) round(x+.5*10^-digits, digits)
   roundDown <- function(x,digits = 3) round(x-.5*10^-digits, digits)
   
@@ -45,8 +51,6 @@ ruleExplorer <- function(x, parameter = NULL) {
   yIndexCached <- "confidence"
   zIndexCached <- "lift"
   
-  #logOutput <- shiny::reactiveVal('Output log')
- 
   ### js cannot handle very large arrays
   itemLabels <- itemLabels(dataset)
   if(length(itemLabels) > 10000) 
@@ -78,88 +82,140 @@ ruleExplorer <- function(x, parameter = NULL) {
   }
   
   ## create Shiny UI and server code
-  shiny::shinyApp(ui = shiny::shinyUI(shiny::fluidPage(
-    #shiny::titlePanel("Association Rules"),
+  shiny::shinyApp(ui = shiny::shinyUI(
     
-    shiny::sidebarLayout(
-      shiny::sidebarPanel(
-        
-        shiny::htmlOutput('numRulesOutput'),
-        shiny::br(),
-        shiny::uiOutput("kSelectInput"),
-        shiny::uiOutput("xAxisSelectInput"),
-        shiny::uiOutput("yAxisSelectInput"),
-        shiny::uiOutput("cAxisSelectInput"),
-        shiny::br(),
-        shiny::sliderInput("supp", "Minimum Support:", min = minSupp, max = maxSupp, value = supp , step = (maxSupp-minSupp)/10000, sep =""),
-        shiny::sliderInput("conf", "Minimum Confidence:", min = minConf, max = maxConf, value = conf , step =  (maxConf-minConf)/1000, sep = ""), 
-        shiny::sliderInput("lift", "Minimum Lift:", min = minLift, max = maxLift, value = lift , step =  (maxLift-minLift)/1000, sep = ""), 
-        shiny::numericInput("minL", "Min. items in rule:", 2), 
-        shiny::numericInput("maxL", "Max. items in rule:", 10), 
-        shiny::br(),
-        shiny::HTML('<b>Filter rules by items:</b>'),
-        shiny::br(),
-        shiny::selectInput('colsType',NULL,c('Exclude items:'='rem','Require items:'='req')),
-        shiny::uiOutput("choose_columns"), 
-        shiny::selectInput('colsLHSType',NULL,c('Exclude items from LHS:'='rem','Require items in LHS:'='req')),
-        shiny::uiOutput("choose_lhs"), 
-        shiny::selectInput('colsRHSType',NULL,c('Exclude items from RHS:'='rem','Require items in RHS:'='req')),
-        shiny::uiOutput("choose_rhs"), 
-        shiny::br(),
-        shiny::downloadButton('rules.csv', 'Download Rules as CSV')
-        #,
-      #  shiny::br(),
-      #  shiny::verbatimTextOutput('logOutput')
-      ),
+    shiny::fluidPage(
+      theme = shinythemes::shinytheme("yeti"),
       
-      shiny::mainPanel(
-        shiny::tabsetPanel(id='mytab',
-          shiny::tabPanel('Data Table', value='datatable', shiny::dataTableOutput("rulesDataTable")),
-          shiny::tabPanel('Scatter', value='scatter', plotlyOutput("scatterPlot", width='100%', height='100%')),
-          shiny::tabPanel('Matrix', value='matrix', plotlyOutput("matrixPlot", width='100%', height='100%')),
-          shiny::tabPanel('Grouped', value='grouped', shiny::plotOutput("groupedPlot", width='100%', height='100%')),
-          shiny::tabPanel('Graph', value='graph', visNetworkOutput("graphPlot", width='100%', height='800px'))
+      title = "Association Rule Explorer",
+      shiny::titlePanel("Association Rule Explorer"),
+      
+      shiny::sidebarLayout(
+        shiny::sidebarPanel(
+          
+          shiny::htmlOutput('numRulesOutput'),
+          shiny::br(),
+          
+          shiny::sliderInput("supp", "Minimum Support:", min = minSupp, max = maxSupp, 
+            value = supp , step = (maxSupp-minSupp)/10000, sep =""),
+          shiny::sliderInput("conf", "Minimum Confidence:", min = minConf, max = maxConf, 
+            value = conf , step =  (maxConf-minConf)/1000, sep = ""), 
+          shiny::sliderInput("lift", "Minimum Lift:", min = minLift, max = maxLift, 
+            value = lift , step =  (maxLift-minLift)/1000, sep = ""), 
+          shiny::sliderInput("length", "Rule length (from-to):", min = 2, max = 20, 
+            value = c(2,10) , step =  1, sep = ""), 
+          
+          #shiny::br(),
+          shiny::em(shiny::HTML('Filter rules by items:')),
+          #shiny::br(),
+          shiny::selectInput('colsType',NULL,c('Exclude items:'='rem','Require items:'='req')),
+          shiny::uiOutput("choose_columns"), 
+          shiny::selectInput('colsLHSType',NULL,c('Exclude items from LHS:'='rem','Require items in LHS:'='req')),
+          shiny::uiOutput("choose_lhs"), 
+          shiny::selectInput('colsRHSType',NULL,c('Exclude items from RHS:'='rem','Require items in RHS:'='req')),
+          shiny::uiOutput("choose_rhs") 
+        ),
+        
+        shiny::mainPanel(
+          shiny::tabsetPanel(id='tabs',
+            
+            shiny::tabPanel('Data Table', value='datatable', 
+              shiny::br(),
+              DT::dataTableOutput("rulesDataTable")
+            ),
+            
+            shiny::tabPanel('Scatter', value='scatter', 
+              shiny::wellPanel(
+                shiny::fluidRow(
+                  shiny::column(3, shiny::uiOutput("xAxisSelectInput")),
+                  shiny::column(3, shiny::uiOutput("yAxisSelectInput")),
+                  shiny::column(3, shiny::uiOutput("cAxisSelectInput")),
+                  shiny::column(3, shiny::sliderInput("max_scatter", "Top rules shown (keep below 500):", 
+                    min = 1, max = length(x), value = min(100, length(x)), step = 1, sep = ""))
+                )),
+              plotly::plotlyOutput("scatterPlot", width='100%', height='100%')
+            ),
+            
+            shiny::tabPanel('Matrix', value='matrix', 
+              shiny::wellPanel(
+                shiny::fluidRow(
+                  shiny::column(6, shiny::uiOutput("cAxisSelectInput_matrix")),
+                  shiny::column(6, shiny::sliderInput("max_matrix", "Top rules shown (keep below 500):", 
+                    min = 1, max = length(x), value = min(100, length(x)), step = 1, sep = ""))
+                )),
+              
+              plotly::plotlyOutput("matrixPlot", width='100%', height='100%')
+            ),
+            
+          shiny::tabPanel('Grouped', value='grouped', 
+            shiny::wellPanel(
+              shiny::fluidRow(
+                shiny::column(6, shiny::uiOutput("kSelectInput")),
+                shiny::column(6, shiny::uiOutput("cAxisSelectInput_grouped"))
+              )),
+            shiny::plotOutput("groupedPlot")
+            # size is specified in renderPlot
+            #, width='80%', height='50%')
+            ),
+            
+            shiny::tabPanel('Graph', value='graph', 
+              shiny::wellPanel(
+                shiny::fluidRow(
+                  shiny::column(6, shiny::uiOutput("cAxisSelectInput_graph")),
+                  shiny::column(6, shiny::sliderInput("max_graph", "Top rules shown (keep below 500):", 
+                    min = 1, max = length(x), value = min(100, length(x)), step = 1, sep = ""))
+                )),
+              
+              visNetwork::visNetworkOutput("graphPlot", width='100%', height='800px')
+            ),
+            
+            shiny::tabPanel('Export', value='export', 
+              shiny::br(), 
+              shiny::downloadButton('rules.csv', 'Export rules (CSV)'))
+          )
         )
-      )
-    ) 
-  )),
-    
+      ) 
+    )),
+   
+     
     server = function(input, output, session) {
       
-      
       output$numRulesOutput <- shiny::renderUI({
-        shiny::HTML(paste('<b>', 'Rules selected: ', length(rules()), '</b>'))
+        shiny::em(shiny::HTML(paste('Selected rules: ', length(rules()))))
       })
-
+      
       output$kSelectInput <- shiny::renderUI({
-        if(input$mytab == 'grouped') {
           shiny::sliderInput('k', label='Choose # of rule clusters', min=1, max=50, step=1, value=15)
-        }
       })
       
       output$xAxisSelectInput <- shiny::renderUI({
-        if(input$mytab == 'scatter') {
           shiny::selectInput("xAxis","X Axis:", colnames(quality(rules())), selected=xIndexCached)
-        }
       })
       
       output$yAxisSelectInput <- shiny::renderUI({
-        if(input$mytab == 'scatter') {
           shiny::selectInput("yAxis","Y Axis:", colnames(quality(rules())), selected=yIndexCached)
-        }
       })
       
       output$cAxisSelectInput <- shiny::renderUI({
-        if(input$mytab %in% c('scatter', 'matrix', 'graph', 'grouped')) {
           shiny::selectInput("cAxis","Shading:", colnames(quality(rules())), selected=zIndexCached)
-        }
+      })
+      
+      output$cAxisSelectInput_matrix <- shiny::renderUI({
+        shiny::selectInput("cAxis_matrix","Shading:", colnames(quality(rules())), selected=zIndexCached)
+      })
+      
+      output$cAxisSelectInput_grouped <- shiny::renderUI({
+        shiny::selectInput("cAxis_grouped","Shading:", colnames(quality(rules())), selected=zIndexCached)
+      })
+      
+      output$cAxisSelectInput_graph <- shiny::renderUI({
+        shiny::selectInput("cAxis_graph","Shading:", colnames(quality(rules())), selected=zIndexCached)
       })
       
       output$choose_columns <- shiny::renderUI({
         shiny::selectizeInput('cols', NULL, itemLabels, multiple = TRUE)
       })
       
-      
       output$choose_lhs <- shiny::renderUI({
         shiny::selectizeInput('colsLHS', NULL, itemLabels, multiple = TRUE)
       })
@@ -168,10 +224,6 @@ ruleExplorer <- function(x, parameter = NULL) {
         shiny::selectizeInput('colsRHS', NULL, itemLabels, multiple = TRUE)
       })
       
-      #output$logOutput <- shiny::renderText({
-      #  logOutput()
-      #})
-      
       ## caching data
       cachedRules <- NULL
       cachedSupp <- supp
@@ -191,31 +243,29 @@ ruleExplorer <- function(x, parameter = NULL) {
       
       ### remine rules if necessary dataset is transactions!
       remineRules <- shiny::reactive({
-       
+        
         ### use a minimum of 1 absolute support!
         supp <- input$supp
         if(supp == 0) supp <- 1/length(dataset)
-         
+        
         rules <- apriori(dataset, parameter = list(
           support = as.numeric(supp), 
           confidence = as.numeric(input$conf), 
-          minlen = input$minL, 
-          maxlen = input$maxL),
+          minlen = input$length[1], 
+          maxlen = input$length[2]),
           control = list(verbose = TRUE))
         quality(rules) <- interestMeasure(rules, transactions = dataset)
         
         message("Remined ", length(rules), " rules.")
-        #lo <- paste(logOutput(),'\nRemined',length(rules),'rules.')
-        #logOutput(lo)
         
         cachedRules <<- rules
         cachedSupp <<- input$supp
         cachedConf <<- input$conf
         cachedLift <<- input$lift
-        cachedMinL <<- input$minL
-        cachedMaxL <<- input$maxL
+        cachedMinL <<- input$length[1]
+        cachedMaxL <<- input$length[2]
       })
-     
+      
       ### handle warning for too low support
       override <- shiny::reactiveVal(FALSE)
       
@@ -224,51 +274,44 @@ ruleExplorer <- function(x, parameter = NULL) {
         # reset the slider (Note this does not change input$supp!)
         shiny::updateSliderInput(session, "supp", value = cachedSupp)
       })
-
+      
       shiny::observeEvent(input$continue, { 
         shiny::removeModal() 
         override(TRUE)
       })
-          
+      
       rules <- shiny::reactive({
         ### recalculate rules?
         
         if(is(dataset, 'transactions')) {
           
-        #  cat("input supp: ", input$supp, "\n")  
-        #  cat("cached supp: ", cachedSupp, "\n")  
-        #  cat("abs supp: ", input$supp*length(dataset), "\n")  
-        #  cat("override: ", override(), "\n")  
-          
           # check for low minimum support first
           if(input$supp*length(dataset) > 10 || override()) {
-        
+            
             if(is.null(cachedRules)) remineRules()
             if((input$supp < cachedSupp) || input$conf < cachedConf) remineRules()
-            if(input$minL < cachedMinL || input$maxL > cachedMaxL) remineRules()
+            if(input$length[1] < cachedMinL || input$length[1] > cachedMaxL) remineRules()
             
           } else { 
-              shiny::showModal(shiny::modalDialog(
-                title='Warning',
-                'Very low minimum support! Too low values can result in long wait times and memory issues.',
-                footer=shiny::tagList(
-                  shiny::actionButton('cancel','cancel'),
-                  shiny::actionButton('continue','proceed')
-                )
-              ))
+            shiny::showModal(shiny::modalDialog(
+              title='Warning',
+              'Very low minimum support! Too low values can result in long wait times and memory issues.',
+              footer=shiny::tagList(
+                shiny::actionButton('cancel','cancel'),
+                shiny::actionButton('continue','proceed')
+              )
+            ))
           }
         }
         
         ar <- cachedRules
-        
+       
+        ### filter rules 
         if(input$supp > cachedSupp) {
           ar <- subset(ar, subset = support > input$supp)
         }
-       
+        
         if(input$conf > cachedConf) {
-          ### FIXME: R CMD check complains about: 
-          ### shiny_arules : <anonymous>: no visible binding for global variable 'confidence' 
-          #ar <- subset(ar, subset = quality(ar)$confidence > input$conf)
           ar <- subset(ar, subset = quality(ar)$confidence > input$conf)
         }
         
@@ -276,12 +319,12 @@ ruleExplorer <- function(x, parameter = NULL) {
           ar <- subset(ar, subset= lift > input$lift)
         }
         
-        if(input$minL > cachedMinL) {
-          ar <- ar[size(ar) >= input$minL]   
+        if(input$length[1] > cachedMinL) {
+          ar <- ar[size(ar) >= input$length[1]]   
         }
         
-        if(input$maxL < cachedMaxL) {
-          ar <- ar[size(ar) <= input$maxL]   
+        if(input$length[2] < cachedMaxL) {
+          ar <- ar[size(ar) <= input$length[2]]   
         }
         
         if(input$colsType == 'rem' && length(input$cols) > 0) {
@@ -292,7 +335,6 @@ ruleExplorer <- function(x, parameter = NULL) {
           ar <- subset(ar, subset=items %in% input$cols)
         }
         
-        
         if(input$colsLHSType == 'rem' && length(input$colsLHS) > 0) {
           ar <- subset(ar, subset=!(lhs %in% input$colsLHS))
         }
@@ -309,24 +351,18 @@ ruleExplorer <- function(x, parameter = NULL) {
           ar <- subset(ar, subset=rhs %in% input$colsRHS)
         }
         
-        
-        ### update lift slider
-        #maxLift <<- ceiling(max(quality(ar)$lift))
-        #shiny::updateSliderInput(session,"lift",value = input$lift, min=minLift, max=maxLift, step = 1/10000)
-        
-        shiny::validate(
-          #shiny::need(length(ar) > 0, "No matching rules found!")
-        )
+        shiny::validate()
         
         ar
-        
-      }
-      )
+      })
       
       # remember settings for other plots
       shiny::observe({ shiny::req(input$xAxis); xIndexCached <<- input$xAxis })
       shiny::observe({ shiny::req(input$yAxis); yIndexCached <<- input$yAxis })
       shiny::observe({ shiny::req(input$cAxis); zIndexCached <<- input$cAxis })
+      shiny::observe({ shiny::req(input$cAxis_matrix); zIndexCached <<- input$cAxis_matrix })
+      shiny::observe({ shiny::req(input$cAxis_grouped); zIndexCached <<- input$cAxis_grouped })
+      shiny::observe({ shiny::req(input$cAxis_graph); zIndexCached <<- input$cAxis_graph })
       
       
       # Present errors nicely to the user
@@ -339,20 +375,20 @@ ruleExplorer <- function(x, parameter = NULL) {
       
       ## Grouped Plot #########################
       output$groupedPlot <- shiny::renderPlot({
-        shiny::req(input$cAxis, input$k)
+        shiny::req(input$cAxis_grouped, input$k)
         handleErrors()
         
-        plot(rules(), method='grouped', shading = input$cAxis, control=list(k=input$k))
-      }, height=800, width=800)
+        plot(rules(), method='grouped', shading = input$cAxis_grouped, control=list(k=input$k))
+      }, height=600, width='auto')
       
       
       ## Graph Plot ##########################
-      output$graphPlot <- renderVisNetwork({
-        shiny::req(input$cAxis)
+      output$graphPlot <- visNetwork::renderVisNetwork({
+        shiny::req(input$cAxis_graph, input$max_graph)
         handleErrors()
         
-        plt <- plot(rules(), method='graph', shading = input$cAxis, engine='htmlwidget')
-        
+        plt <- plot(rules(), method='graph', shading = input$cAxis_graph, engine='htmlwidget',
+          control = list(max=input$max_graph))
         plt$sizingPolicy <- htmlwidgets::sizingPolicy(
           viewer.paneHeight=1000,
           browser.defaultHeight=1000,
@@ -367,47 +403,35 @@ ruleExplorer <- function(x, parameter = NULL) {
       
       
       ## Scatter Plot ##########################
-      output$scatterPlot <- renderPlotly({
-        shiny::req(input$xAxis, input$yAxis, input$cAxis)
+      output$scatterPlot <- plotly::renderPlotly({
+        shiny::req(input$xAxis, input$yAxis, input$cAxis, input$max_scatter)
         handleErrors()
-        
-        .plotly_arules(rules(), method = 'scatterplot',
-          measure=c(input$xAxis, input$yAxis), shading = input$cAxis)
+        plot(rules(), method = 'scatterplot',
+          measure=c(input$xAxis, input$yAxis), shading = input$cAxis, engine = 'htmlwidget',
+          control = list(max=input$max_scatter))
       })
       
       
-      ## Parallel Coordinates Plot ###################
-      output$paracoordPlot <- shiny::renderPlot({
-        handleErrors()
-        
-        plot(rules(), method='paracoord')
-      }, height=800, width=800)
-      
-      
       ## Matrix Plot ###################
-      output$matrixPlot <- renderPlotly({
+      output$matrixPlot <- plotly::renderPlotly({
+        shiny::req(input$cAxis_matrix, input$max_matrix)
         handleErrors()
-        
-        .plotly_arules(rules(), method='matrix', shading=input$cAxis)
+        plot(rules(), method = 'matrix', shading = input$cAxis_matrix, engine = 'htmlwidget',
+          control = list(max=input$max_matrix))
       })
       
-      ## Rules Data Table ##########################
-      output$rulesDataTable <- shiny::renderDataTable({
+      ## Data Table ##########################
+      output$rulesDataTable <- DT::renderDataTable({
         handleErrors()
-        
-        
-        x <- rules()
-        data.frame(LHS = labels(lhs(x)), RHS = labels(rhs(x)), quality(x))
-      }, options = list(filter = "top", rownames = paste0('[', 1:length(x), ']'))
+        inspectDT(rules())
+      }
       )
       
-      ## Download data to csv ########################
+      ## Export ########################
       output$rules.csv <- shiny::downloadHandler(
         filename = 'rules.csv',
         content = function(file) { 
-          x <- rules()
-          data <- data.frame(LHS = labels(lhs(x)), RHS = labels(rhs(x)), quality(x))
-          write.csv(data, file) 
+          write.csv(as(rules(), "data.frame"), file) 
         }
       )
       
